#include <Window.h>
#include <Message.h>
#include <View.h>
#include <StringView.h>
#include <TranslationUtils.h>
#include <Box.h>
#include <String.h>
#include <Bitmap.h>
#include "InputView.h"
#include <stdio.h>
#include "smskeynames.h"

struct input_lister
{
	const char *label;
	const char *text;
} inp_list[] = {
	{ "Up:  ", "Enter The Key For Up" },
	{ "Down:  ", "Enter The Key For Down" },
	{ "Left:  ", "Enter The Key For Left" },
	{ "Right:  ", "Enter The Key For Right" },
	{ "Button1:  ", "Enter The Key For button1" },
	{ "Button2:  ", "Enter The Key For button2" },
	{ "Start:  ", "Enter The Key For start" },
	{ 0, 0 }
};

InputView::InputView(BRect rect, BWindow *front)
	: BView(rect, "", B_FOLLOW_ALL, B_WILL_DRAW | B_PULSE_NEEDED),
	key(0),
	input_num(0),
	input(false),
	tile_bitmap(0)
{
	this->front = front;
	tile_bitmap = BTranslationUtils::GetBitmap("tile_bmap");
	
	box = new BBox(BRect(10, 10, 210, 210));
	box->SetLabel("Keys:");
	AddChild(box);
	
	input_sviews[UP] = new BStringView(BRect(10, 20, 100, 35), "up_input", "Up:");
	box->AddChild(input_sviews[UP]);
	
	input_sviews[DOWN] = new BStringView(BRect(10, 40, 100, 55), "down_input", "Down:");
	box->AddChild(input_sviews[DOWN]);
	
	input_sviews[LEFT] = new BStringView(BRect(10, 60, 100, 75), "left_input", "Left:");
	box->AddChild(input_sviews[LEFT]);
	
	input_sviews[RIGHT] = new BStringView(BRect(10, 80, 100, 95), "right_input", "Right:");
	box->AddChild(input_sviews[RIGHT]);
	
	input_sviews[BUTTON1] = new BStringView(BRect(10, 100, 100, 115), "button1_input", "Button1:");
	box->AddChild(input_sviews[BUTTON1]);
	
	input_sviews[BUTTON2] = new BStringView(BRect(10, 120, 100, 135), "button2_input", "Button2:");
	box->AddChild(input_sviews[BUTTON2]);
	
	input_sviews[START] = new BStringView(BRect(10, 140, 100, 155), "start_input", "Start:");
	box->AddChild(input_sviews[START]);
	
	sview = new BStringView(BRect(10, 160, 160, 185), "label", inp_list[0].text);
	sview->SetFontSize(12);
	
	box->AddChild(sview);
}

void InputView::AttachedToWindow()
{
	SetViewBitmap(tile_bitmap);
}

void InputView::MessageReceived(BMessage *m)
{
	int32 key_code;
	switch(m->what)
	{
		case B_UNMAPPED_KEY_DOWN:
			if(m->FindInt32("key", &key_code) == B_NO_ERROR)
				key = key_code;
		break;
		default:
			BView::MessageReceived(m);
		break;
	}
}

void InputView::KeyDown(const char *bytes, int32 numBytes)
{
	BMessage *msg = front->CurrentMessage();
	if(msg)
	{
		int32 key_code;
		if(msg->FindInt32("key", &key_code) == B_NO_ERROR)
		{
			key = key_code;
		}
	}
	BView::KeyDown(bytes, numBytes);
}

void InputView::Pulse()
{
	Window()->Lock();
	Draw(Bounds());
	Window()->Unlock();
}

void InputView::Draw(BRect updateRect)
{
	sview->StrokeRect(BRect(0, 5, 150, 25));
	if(IsInput()) {
		if(key == 1) { key = 0; Window()->PostMessage(B_QUIT_REQUESTED); }
		if(key) { which[input_num] = key; ChangeLabel(input_num, key); input_num++; sview->SetText(inp_list[input_num].text); }
		if(key) { key = 0; }
		if(input_num >= MAX) { input_num = 0; sview->SetText(inp_list[input_num].text); }
	}
}

void InputView::ChangeLabel(int index, int key)
{
	BString string(inp_list[index].label);
	string << key_name(key);
	input_sviews[index]->SetText(string.String());
}

void InputView::SetKeys(uint8 *which)
{
	this->which = which;
	for(int i = 0; i < MAX; i++) {
		ChangeLabel(i, which[i]);
	}
}

InputView::~InputView()
{
	if(tile_bitmap) delete tile_bitmap;
}
