#include <Message.h>
#include <File.h>
#include <Path.h>
#include <string.h>
#include <FindDirectory.h>
#include <stdio.h>
#include "smssettings.h"
#include "smskey.h"

void SetDefault(smssettings &settings)
{
	// Ui
	settings.uisettings.mode = 0;
	settings.uisettings.mute = 1;
	
	// KeyBoard 0
	settings.keypad0.kup = SMS_UP;
	settings.keypad0.kdown = SMS_DOWN;
	settings.keypad0.kleft = SMS_LEFT;
	settings.keypad0.kright = SMS_RIGHT;
	settings.keypad0.kbutton1 = SMS_Z;
	settings.keypad0.kbutton2 = SMS_X;
	settings.keypad0.kstart = SMS_RETURN;
	
	// KeyBoard 1
	settings.keypad1.kup = SMS_Y;
	settings.keypad1.kdown = SMS_B;
	settings.keypad1.kleft = SMS_G;
	settings.keypad1.kright = SMS_J;
	settings.keypad1.kbutton1 = SMS_B;
	settings.keypad1.kbutton2 = SMS_N;
	settings.keypad1.kstart = SMS_M;
	
	// Joypad 0
	//settings.joypad0.stick_index = 0;
	settings.joypad0.jbutton1 = 0;
	settings.joypad0.jbutton2 = 1;
	settings.joypad0.jstart = 2;
	settings.joypad0.swapy = 0;
	
	// Joypad 1
	//settings.joypad1.stick_index = 1;
	settings.joypad1.jbutton1 = 0;
	settings.joypad1.jbutton2 = 1;
	settings.joypad1.jstart = 2;
	settings.joypad1.swapy = 0;
}

void GetSettings(smssettings &settings)
{
	BPath path;
	BMessage smsplus;
	if(find_directory(B_USER_SETTINGS_DIRECTORY, &path, true) != B_OK)
	path.SetTo("/boot/home/config/settings");
	path.Append("sms_settings");
	
	BFile settings_file(path.Path(), B_READ_ONLY);
	if(settings_file.InitCheck() != B_OK) {
		SetDefault(settings);
		return;
	}
	
	smsplus.Unflatten(&settings_file);
	
	// Ui
	smsplus.FindInt8("mode", (int8 *)&settings.uisettings.mode);
	smsplus.FindInt8("mute", (int8 *)&settings.uisettings.mute);
	
	// Keyboard 0
	smsplus.FindInt8("kup0", (int8 *)&settings.keypad0.kup);
	smsplus.FindInt8("kdown0", (int8 *)&settings.keypad0.kdown);
	smsplus.FindInt8("kleft0", (int8 *)&settings.keypad0.kleft);
	smsplus.FindInt8("kright0", (int8 *)&settings.keypad0.kright);
	smsplus.FindInt8("kbutton10", (int8 *)&settings.keypad0.kbutton1);
	smsplus.FindInt8("kbutton20", (int8 *)&settings.keypad0.kbutton2);
	smsplus.FindInt8("kstart0", (int8 *)&settings.keypad0.kstart);
	
	// Keyboard 1
	smsplus.FindInt8("kup1", (int8 *)&settings.keypad1.kup);
	smsplus.FindInt8("kdown1", (int8 *)&settings.keypad1.kdown);
	smsplus.FindInt8("kleft1", (int8 *)&settings.keypad1.kleft);
	smsplus.FindInt8("kright1", (int8 *)&settings.keypad1.kright);
	smsplus.FindInt8("kbutton11", (int8 *)&settings.keypad1.kbutton1);
	smsplus.FindInt8("kbutton21", (int8 *)&settings.keypad1.kbutton2);
	smsplus.FindInt8("kstart1", (int8 *)&settings.keypad1.kstart);
	
	// Joypad 0
	smsplus.FindInt8("jbutton10", (int8 *)&settings.joypad0.jbutton1);
	smsplus.FindInt8("jbutton20", (int8 *)&settings.joypad0.jbutton2);
	smsplus.FindInt8("jstart0", (int8 *)&settings.joypad0.jstart);
	smsplus.FindInt8("jswapy0", (int8 *)&settings.joypad0.swapy);
	
	// Joypad 1
	smsplus.FindInt8("jbutton11", (int8 *)&settings.joypad1.jbutton1);
	smsplus.FindInt8("jbutton21", (int8 *)&settings.joypad1.jbutton2);
	smsplus.FindInt8("jstart1", (int8 *)&settings.joypad1.jstart);
	smsplus.FindInt8("jswapy1", (int8 *)&settings.joypad1.swapy);
	
	smsplus.Flatten(&settings_file);
}

static inline status_t SaveInt8(BMessage &msg, const char *string, int8 data)
{
	if(msg.ReplaceInt8(string, data) == B_NAME_NOT_FOUND)
		return msg.AddInt8(string, data);
	return msg.ReplaceInt8(string, data);
}

void SetSettings(const smssettings &settings)
{
	BPath path;
	BMessage smsplus;
	if(find_directory(B_USER_SETTINGS_DIRECTORY, &path, true) != B_OK)
	path.SetTo("/boot/home/config/settings");
	path.Append("sms_settings");
	
	BFile settings_file(path.Path(), B_READ_WRITE | B_CREATE_FILE);
	smsplus.Unflatten(&settings_file);
	
	// Ui
	SaveInt8(smsplus, "mode", settings.uisettings.mode);
	SaveInt8(smsplus, "mute", settings.uisettings.mute);
	
	// Keyboard 0
	SaveInt8(smsplus, "kup0", settings.keypad0.kup);
	SaveInt8(smsplus, "kdown0", settings.keypad0.kdown);
	SaveInt8(smsplus, "kleft0", settings.keypad0.kleft);
	SaveInt8(smsplus, "kright0", settings.keypad0.kright);
	SaveInt8(smsplus, "kbutton10", settings.keypad0.kbutton1);
	SaveInt8(smsplus, "kbutton20", settings.keypad0.kbutton2);
	SaveInt8(smsplus, "kstart0", settings.keypad0.kstart);
	
	// Keyboard 1
	SaveInt8(smsplus, "kup1", settings.keypad1.kup);
	SaveInt8(smsplus, "kdown1", settings.keypad1.kdown);
	SaveInt8(smsplus, "kleft1", settings.keypad1.kleft);
	SaveInt8(smsplus, "kright1", settings.keypad1.kright);
	SaveInt8(smsplus, "kbutton11", settings.keypad1.kbutton1);
	SaveInt8(smsplus, "kbutton21", settings.keypad1.kbutton2);
	SaveInt8(smsplus, "kstart1", settings.keypad1.kstart);
	
	// Joypad 0
	SaveInt8(smsplus, "jbutton10", settings.joypad0.jbutton1);
	SaveInt8(smsplus, "jbutton20", settings.joypad0.jbutton2);
	SaveInt8(smsplus, "jstart0", settings.joypad0.jstart);
	SaveInt8(smsplus, "jswapy0", settings.joypad0.swapy);
	
	// Joypad 1
	SaveInt8(smsplus, "jbutton11", settings.joypad1.jbutton1);
	SaveInt8(smsplus, "jbutton21", settings.joypad1.jbutton2);
	SaveInt8(smsplus, "jstart1", settings.joypad1.jstart);
	SaveInt8(smsplus, "jswapy1", settings.joypad1.swapy);
	
	settings_file.SetSize(0);
	settings_file.Seek(0, SEEK_SET);
	smsplus.Flatten(&settings_file);
}

